package com.foreverwin.mesnac.dispatch.controller;

import com.foreverwin.modular.core.util.R;
import com.foreverwin.modular.core.util.FrontPage;
import com.foreverwin.modular.core.util.CommonMethods;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import com.foreverwin.mesnac.dispatch.service.ItemBatchService;
import com.foreverwin.mesnac.dispatch.model.ItemBatch;
import java.util.List;

/**
 *
 * @author Leon.L
 * @since 2021-06-18
 */
@RestController
@RequestMapping("/Z-ITEM-BATCH")
public class ItemBatchController {

    @Autowired
    public ItemBatchService itemBatchService;

    /**
    * 根据id查询
    *
    * @param id 主键
    * @return
    */
    @ResponseBody
    @GetMapping("/{id:.+}")
    public R getItemBatchById(@PathVariable String id) {
        return R.ok( itemBatchService.getById(id));
    }

    /**
     * 查询所有数据
     *
     * @return
     */
    @ResponseBody
    @GetMapping("")
    public R getItemBatchList(ItemBatch itemBatch){
        List<ItemBatch> result;
        QueryWrapper<ItemBatch> queryWrapper = new QueryWrapper<>();
        queryWrapper.setEntity(itemBatch);
        result = itemBatchService.list(queryWrapper);
        return R.ok(result);
    }

    /**
     * 分页查询数据
     *
     * @param frontPage  分页信息
     * @return
     */
    @ResponseBody
    @GetMapping("/page")
    public R page(FrontPage<ItemBatch> frontPage, ItemBatch itemBatch){
        IPage result;
        QueryWrapper<ItemBatch> queryWrapper = new QueryWrapper<>();
        queryWrapper.setEntity(itemBatch);
        if (frontPage.getGlobalQuery() != null && !"".equals(frontPage.getGlobalQuery().trim())) {
            //TODO modify global query
            queryWrapper.lambda().and(wrapper -> wrapper
                .like(ItemBatch::getHandle, frontPage.getGlobalQuery())
                .or().like(ItemBatch::getSite, frontPage.getGlobalQuery())
                .or().like(ItemBatch::getFactory, frontPage.getGlobalQuery())
                .or().like(ItemBatch::getItem, frontPage.getGlobalQuery())
                .or().like(ItemBatch::getBatch, frontPage.getGlobalQuery())
                .or().like(ItemBatch::getSupplier, frontPage.getGlobalQuery())
                .or().like(ItemBatch::getLabel, frontPage.getGlobalQuery())
                .or().like(ItemBatch::getIsPrint, frontPage.getGlobalQuery())
                .or().like(ItemBatch::getCreateUser, frontPage.getGlobalQuery())
                .or().like(ItemBatch::getModifyUser, frontPage.getGlobalQuery())
    );
        }
        result = itemBatchService.page(frontPage.getPagePlus(), queryWrapper);
        return R.ok(result);
    }

    /**
     * 新增
     * @param itemBatch  传递的实体
     * @return  null 失败  实体成功
     */
    @PostMapping
    public R save(@RequestBody ItemBatch itemBatch) {
        return R.ok(itemBatchService.save(itemBatch));
    }

    /**
     * 修改
     * @param itemBatch  传递的实体
     * @return  null 失败  实体成功
     */
    @PutMapping
    public R updateById(@RequestBody ItemBatch itemBatch) {
        return R.ok(itemBatchService.updateById(itemBatch));
    }

    /**
     * 根据id删除对象
     * @param id  实体ID
     * @return 0 失败  1 成功
     */
    @ResponseBody
    @RequestMapping(method = RequestMethod.DELETE, value = "/{id:.+}")
    public R removeById(@PathVariable("id") String id){
        return R.ok(itemBatchService.removeById(id));
    }

    /**
     * 批量删除对象
     * @param ids 实体集合ID
     * @return  0 失败  1 成功
     */
    @ResponseBody
    @RequestMapping(method = RequestMethod.POST, value = "/delete-batch")
    public R removeByIds(List<String> ids){
        return R.ok(itemBatchService.removeByIds(ids));
    }
}