package com.foreverwin.mesnac.quality.controller;

import com.foreverwin.modular.core.util.R;
import com.foreverwin.modular.core.util.FrontPage;
import com.foreverwin.modular.core.util.CommonMethods;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import com.foreverwin.mesnac.quality.service.InspectionProjectDetailService;
import com.foreverwin.mesnac.quality.model.InspectionProjectDetail;
import java.util.List;

/**
 *
 * @author Robert
 * @since 2021-06-03
 */
@RestController
@RequestMapping("/Z-INSPECTION-PROJECT-DETAIL")
public class InspectionProjectDetailController {

    @Autowired
    public InspectionProjectDetailService inspectionProjectDetailService;

    /**
    * 根据id查询
    *
    * @param id 主键
    * @return
    */
    @ResponseBody
    @GetMapping("/{id:.+}")
    public R getInspectionProjectDetailById(@PathVariable String id) {
        return R.ok( inspectionProjectDetailService.getById(id));
    }

    /**
     * 查询所有数据
     *
     * @return
     */
    @ResponseBody
    @GetMapping("")
    public R getInspectionProjectDetailList(InspectionProjectDetail inspectionProjectDetail){
        List<InspectionProjectDetail> result;
        QueryWrapper<InspectionProjectDetail> queryWrapper = new QueryWrapper<>();
        queryWrapper.setEntity(inspectionProjectDetail);
        result = inspectionProjectDetailService.list(queryWrapper);
        return R.ok(result);
    }

    /**
     * 分页查询数据
     *
     * @param frontPage  分页信息
     * @return
     */
    @ResponseBody
    @GetMapping("/page")
    public R page(FrontPage<InspectionProjectDetail> frontPage, InspectionProjectDetail inspectionProjectDetail){
        IPage result;
        QueryWrapper<InspectionProjectDetail> queryWrapper = new QueryWrapper<>();
        queryWrapper.setEntity(inspectionProjectDetail);
        if (frontPage.getGlobalQuery() != null && !"".equals(frontPage.getGlobalQuery().trim())) {
            //TODO modify global query
            queryWrapper.lambda().and(wrapper -> wrapper
                .like(InspectionProjectDetail::getHandle, frontPage.getGlobalQuery())
                .or().like(InspectionProjectDetail::getInspectProjectBo, frontPage.getGlobalQuery())
                .or().like(InspectionProjectDetail::getParamNo, frontPage.getGlobalQuery())
                .or().like(InspectionProjectDetail::getDescription, frontPage.getGlobalQuery())
                .or().like(InspectionProjectDetail::getInspectMethod, frontPage.getGlobalQuery())
                .or().like(InspectionProjectDetail::getParamType, frontPage.getGlobalQuery())
                .or().like(InspectionProjectDetail::getParamUnit, frontPage.getGlobalQuery())
                .or().like(InspectionProjectDetail::getStandardValue, frontPage.getGlobalQuery())
                .or().like(InspectionProjectDetail::getFalseValue, frontPage.getGlobalQuery())
                .or().like(InspectionProjectDetail::getTrueValue, frontPage.getGlobalQuery())
                .or().like(InspectionProjectDetail::getIsRequired, frontPage.getGlobalQuery())
                .or().like(InspectionProjectDetail::getCreateUser, frontPage.getGlobalQuery())
                .or().like(InspectionProjectDetail::getModifyUser, frontPage.getGlobalQuery())
    );
        }
        result = inspectionProjectDetailService.page(frontPage.getPagePlus(), queryWrapper);
        return R.ok(result);
    }

    /**
     * 新增
     * @param inspectionProjectDetail  传递的实体
     * @return  null 失败  实体成功
     */
    @PostMapping
    public R save(@RequestBody InspectionProjectDetail inspectionProjectDetail) {
        return R.ok(inspectionProjectDetailService.save(inspectionProjectDetail));
    }

    /**
     * 修改
     * @param inspectionProjectDetail  传递的实体
     * @return  null 失败  实体成功
     */
    @PutMapping
    public R updateById(@RequestBody InspectionProjectDetail inspectionProjectDetail) {
        return R.ok(inspectionProjectDetailService.updateById(inspectionProjectDetail));
    }

    /**
     * 根据id删除对象
     * @param id  实体ID
     * @return 0 失败  1 成功
     */
    @ResponseBody
    @RequestMapping(method = RequestMethod.DELETE, value = "/{id:.+}")
    public R removeById(@PathVariable("id") String id){
        return R.ok(inspectionProjectDetailService.removeById(id));
    }

    /**
     * 批量删除对象
     * @param ids 实体集合ID
     * @return  0 失败  1 成功
     */
    @ResponseBody
    @RequestMapping(method = RequestMethod.POST, value = "/delete-batch")
    public R removeByIds(List<String> ids){
        return R.ok(inspectionProjectDetailService.removeByIds(ids));
    }
}